

/**
* This file is part of license combination gpl version 3 license and eCos.
* The corresponding license terms are below.
*
* gpl version 3 Licence:
*
* The file were developed during the student thesis "Data Aggregation 
* using TinyIPFIX in Wireless Sensor Networks" of Benjamin Ertl and is 
* included in the dissertation "Secure Data Transmission in Wireless 
* Sensor Networks" by Corinna Schmitt during employment at the Technische
* Universitt Mnchen, Department Computer Science, Chair Network 
* Architectures and Services (Germany).
* Copyright (C) 2013  
* Authors: Benjamin Ertl and Corinna Schmitt (schmitt[at]net.in.tum.de)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
*
* eCos Licence: (http://ecos.sourceware.org)
*
* This file is part of eCos, the Embedded Configurable Operating System. 
* Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 
* 2007, 2008, 2009 Free Software Foundation, Inc. 
*
* eCos is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free 
* Software Foundation; either version 3 any later version. 
* 
* eCos is distributed in the hope that it will be useful, but WITHOUT ANY 
* WARRANTY; without even the implied warranty of MERCHANTABILITY or 
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
* more details. 
* 
* You should have received a copy of the GNU General Public License along 
* with eCos; if not, write to the Free Software Foundation, Inc., 51 
* Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
*
* As a special exception, if other files instantiate templates or use 
* macros or inline functions from this file, or you compile this file and * link it with other works to produce a work based on this file, this file 
* does not by itself cause the resulting work to be covered by the GNU 
* General Public License. However the source code for this file must still 
* be made available in accordance with section (3) of the GNU General 
* Public License v2. 
* 
* This exception does not invalidate any other reasons why a work based on 
* this file might be covered by the GNU General Public License.
*
*/


package ipfix;

import java.util.*;

import xml.MetadataHeap;

public class Template {
	private Hashtable<String, Field> fieldshash;
	private LinkedList<Field> fieldslist;
	int ID, fieldcount, sequence;
	String source;
	long observation;
	
	public Template(int ID, String source, long observation){
		fieldshash = new Hashtable<String, Field>();
		fieldslist = new LinkedList<Field>();
		this.ID = ID;
		this.source = source;
		this.observation = observation;
	}
		
	public Field getField(int ID, long enterpriseID){
		Field current = null;
		if (0x8000 >= ID){ 
			current = fieldshash.get(ID + "|" + enterpriseID);
		}else{
			current = fieldshash.get(ID + "");
		}
		return current;
	}
	
	public void addField(Field field){
		if(getField(field.getID(), field.getEnterpriseID()) == null){
			fieldcount++;
			fieldslist.addLast(field);
		}else{
			int index = fieldslist.indexOf(field);
			fieldslist.remove(index);
			fieldslist.add(index, field);
		}
		
		if (0x8000 >= field.getID()){
			fieldshash.put(field.getID() + "|" + field.getEnterpriseID(), field);
		}else{
			fieldshash.put(field.getID() + "", field);
		}
		
	}
	
	public LinkedList<Field> getFields(){
		return fieldslist;
	}
	
	public int getSequence() {
		return sequence;
	}
	
	public void setSequence(int sequence) {
		this.sequence = sequence;
	}
	
	public void updateAllFields(){
		for (Field curr : fieldslist) {
			MetadataHeap.getInstance().updateField(ID,source,observation,curr);
		}
	}
	
	public String getQualifier(){
		return ID+"|"+source+"|"+observation;
	}
}
