

/**
* This file is part of license combination gpl version 3 license and eCos.
* The corresponding license terms are below.
*
* gpl version 3 Licence:
*
* The file were developed during the student thesis "Data Aggregation 
* using TinyIPFIX in Wireless Sensor Networks" of Benjamin Ertl and is 
* included in the dissertation "Secure Data Transmission in Wireless 
* Sensor Networks" by Corinna Schmitt during employment at the Technische
* Universitt Mnchen, Department Computer Science, Chair Network 
* Architectures and Services (Germany).
* Copyright (C) 2013  
* Authors: Benjamin Ertl and Corinna Schmitt (schmitt[at]net.in.tum.de)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
*
* eCos Licence: (http://ecos.sourceware.org)
*
* This file is part of eCos, the Embedded Configurable Operating System. 
* Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 
* 2007, 2008, 2009 Free Software Foundation, Inc. 
*
* eCos is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free 
* Software Foundation; either version 3 any later version. 
* 
* eCos is distributed in the hope that it will be useful, but WITHOUT ANY 
* WARRANTY; without even the implied warranty of MERCHANTABILITY or 
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
* more details. 
* 
* You should have received a copy of the GNU General Public License along 
* with eCos; if not, write to the Free Software Foundation, Inc., 51 
* Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
*
* As a special exception, if other files instantiate templates or use 
* macros or inline functions from this file, or you compile this file and * link it with other works to produce a work based on this file, this file 
* does not by itself cause the resulting work to be covered by the GNU 
* General Public License. However the source code for this file must still 
* be made available in accordance with section (3) of the GNU General 
* Public License v2. 
* 
* This exception does not invalidate any other reasons why a work based on 
* this file might be covered by the GNU General Public License.
*
*/


package listeners;

import java.io.IOException;
import java.net.*;

/**
 * Starts a UDP Socket on the specified Port in a new Thread.
 * The read packets are passed to the IPFIX Parser. When a package
 * without payload is recieved, the Socket is closed and the Thread ends.
 * 
 * @author Thomas Kothmayr
 * @version 0.1
 */

public class UDPListener implements Runnable{

	private DatagramSocket serverSocket;
	private boolean running = true;
	
	public UDPListener(int port){
		try {
			serverSocket = new DatagramSocket(port);
			new Thread(this).start();
		} catch (SocketException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void run() {
		byte[] buf = new byte[2048];
		
		while(running){
			DatagramPacket p = new DatagramPacket(buf, buf.length);
			try {
				serverSocket.receive(p);
			} catch (IOException e) {
				serverSocket.close();
				e.printStackTrace();
			}
			InetAddress IPAddress = p.getAddress();
            int senderport = p.getPort();
            
            byte[] data = p.getData();
            byte[] realdata = new byte[p.getLength()];
            System.arraycopy(data, 0, realdata, 0, realdata.length);
            if(data.length > 4){
            	try {
					ipfix.IPFIXParser.parse(realdata, "["+IPAddress.getHostAddress()+"]:"+senderport);
				} catch (Exception e) {
					e.printStackTrace();
				}
            }else if (data.length == 0){
            	running = false;
            }
		}		
	}
	
	public void kill(){
		serverSocket.close();
		running = false;
	}
	
}
