

/**
* This file is part of license combination gpl version 3 license and eCos.
* The corresponding license terms are below.
*
* gpl version 3 Licence:
*
* The file were developed during the student thesis "Data Aggregation 
* using TinyIPFIX in Wireless Sensor Networks" of Benjamin Ertl and is 
* included in the dissertation "Secure Data Transmission in Wireless 
* Sensor Networks" by Corinna Schmitt during employment at the Technische
* Universitt Mnchen, Department Computer Science, Chair Network 
* Architectures and Services (Germany).
* Copyright (C) 2013  
* Authors: Benjamin Ertl and Corinna Schmitt (schmitt[at]net.in.tum.de)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
*
* eCos Licence: (http://ecos.sourceware.org)
*
* This file is part of eCos, the Embedded Configurable Operating System. 
* Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 
* 2007, 2008, 2009 Free Software Foundation, Inc. 
*
* eCos is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free 
* Software Foundation; either version 3 any later version. 
* 
* eCos is distributed in the hope that it will be useful, but WITHOUT ANY 
* WARRANTY; without even the implied warranty of MERCHANTABILITY or 
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
* more details. 
* 
* You should have received a copy of the GNU General Public License along 
* with eCos; if not, write to the Free Software Foundation, Inc., 51 
* Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
*
* As a special exception, if other files instantiate templates or use 
* macros or inline functions from this file, or you compile this file and * link it with other works to produce a work based on this file, this file 
* does not by itself cause the resulting work to be covered by the GNU 
* General Public License. However the source code for this file must still 
* be made available in accordance with section (3) of the GNU General 
* Public License v2. 
* 
* This exception does not invalidate any other reasons why a work based on 
* this file might be covered by the GNU General Public License.
*
*/


package xml;

import ipfix.Field;
import ipfix.TemplateHeap;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Stores additional metadata about IPFIX Fields in a Hashtable and provides a method for
 * updating existing instances of Field with the metadata that is relevant to it. 
 * 
 * @author Thomas Kothmayr
 * @version 0.1
 */

public class MetadataHeap {
	private static MetadataHeap instance = null;
	private Hashtable<String, Field> heap;

	private MetadataHeap(){
		heap = new Hashtable<String, Field>();
	}
	
	public void initFromXML(File file, DefaultHandler handler) throws SAXException, IOException, ParserConfigurationException{
		heap = new Hashtable<String, Field>();
		SAXParserFactory.newInstance().newSAXParser().parse(file, handler);
		TemplateHeap.getInstance().updateAllTemplates();
	}
	
	public void initFromXML(String xml, DefaultHandler handler) throws SAXException, IOException, ParserConfigurationException{
		heap = new Hashtable<String, Field>();
		SAXParserFactory.newInstance().newSAXParser().parse(new ByteArrayInputStream(xml.getBytes()), handler);
		TemplateHeap.getInstance().updateAllTemplates();
	}
	
	public static synchronized MetadataHeap getInstance(){
		if (instance == null) instance = new MetadataHeap();
		return instance;
	}

	public synchronized void putFields(String qualifier, Field field) throws Exception{
		Field current = heap.get(qualifier+"|"+field.getID()+"|"+field.getEnterpriseID());
		if(current == null)	heap.put(qualifier+"|"+field.getID()+"|"+field.getEnterpriseID(), field); else current = field;
		return;
	}

	public void updateField(int ID, String source, long observation, Field field){
		synchronized (TemplateHeap.getInstance()) {

			Field current = heap.get("*|*|*|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get(ID+"|*|*|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get("*|"+source+"|*|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get("*|*|"+observation+"|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get(ID+"|"+source+"|*|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get(ID+"|*|"+observation+"|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get("*|"+source+"|"+observation+"|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);

			current = heap.get(ID+"|"+source+"|"+observation+"|"+field.getID()+"|"+field.getEnterpriseID());
			if(current != null) copyMetadata(current, field);
		}
	}

	private void copyMetadata(Field a, Field b){
		b.setDatatype(a.getDatatype());
		b.setName(a.getName());
		b.setConnector(a.getConnector());
		b.setExpression(a.getExpression());
	}
}
