

/**
* This file is part of license combination gpl version 3 license and eCos.
* The corresponding license terms are below.
*
* gpl version 3 Licence:
*
* The file were developed during the student thesis "Datensammlung in Wireless 
* Sensor Networks fuer Autonomic Home Networking of Thomas Kothmayr and is 
* included in the dissertation "Secure Data Transmission in Wireless 
* Sensor Networks" by Corinna Schmitt during employment at the Technische
* Universitt Mnchen, Department Computer Science, Chair Network 
* Architectures and Services (Germany).
* Copyright (C) 2013  
* Authors: Thomas Kothmayr and Corinna Schmitt (schmitt[at]net.in.tum.de)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
* eCos Licence: (http://ecos.sourceware.org)
*
* This file is part of eCos, the Embedded Configurable Operating System. 
* Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 
* 2007, 2008, 2009 Free Software Foundation, Inc. 
*
* eCos is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free 
* Software Foundation; either version 3 any later version. 
* 
* eCos is distributed in the hope that it will be useful, but WITHOUT ANY 
* WARRANTY; without even the implied warranty of MERCHANTABILITY or 
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
* more details. 
* 
* You should have received a copy of the GNU General Public License along 
* with eCos; if not, write to the Free Software Foundation, Inc., 51 
* Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
*
* As a special exception, if other files instantiate templates or use 
* macros or inline functions from this file, or you compile this file and * link it with other works to produce a work based on this file, this file 
* does not by itself cause the resulting work to be covered by the GNU 
* General Public License. However the source code for this file must still 
* be made available in accordance with section (3) of the GNU General 
* Public License v2. 
* 
* This exception does not invalidate any other reasons why a work based on 
* this file might be covered by the GNU General Public License.
*
*/
package ipfix;
import java.io.*;
import java.util.LinkedList;

import xml.FieldMetadataHeap;

public class Parser {
	
	public static void parse(byte[] packet, String source) throws Exception{
		DataInputStream stream = new DataInputStream(new ByteArrayInputStream(packet));
		
		if(stream.available() > 16){
			stream.skip(2);
			int len = stream.readShort();
			if(packet.length < len){
				throw new Exception("Package too short to contain the payload");
			}
			byte[] buff = new byte[len];
			
			stream.skip(12);
			int setID = stream.readShort();
			if(setID == 2){
				stream.close();
				System.arraycopy(packet, 0, buff, 0, len);
				parseTemplate(buff, source);
			}else{
				stream.close();
				System.arraycopy(packet, 0, buff, 0, len);
				parseData(buff, source);
			}
		}
	}
	
	public static void parseTemplate(byte[] packet, String source)throws Exception{
		int validIPFIXHeader = 0x000a;
		int lastvalue = 0;
		Template currentTemplate = null;
		DataInputStream stream = new DataInputStream(new ByteArrayInputStream(packet));

		if(stream.available() > 23 && (lastvalue = stream.readShort()) == validIPFIXHeader){
			if((lastvalue = stream.readShort()) < packet.length){
				throw new Exception("The specified length does not match the length of the packet. It should be " + lastvalue + " but it is "+ packet.length+ " Byte");
			}
			
			int export = stream.readInt();
			int sequence = stream.readInt();
			int observation = stream.readInt();
			
			System.err.printf("Read template package with parameters version %d, length %d, export time %d, sequence %d, observation domain %d\n",10, lastvalue, export, sequence, observation);

			while(stream.available() > 0){
				if((lastvalue = stream.readShort()) != 2){
					throw new Exception("Template sets must have a Set ID of 2, this one has "+ lastvalue+ "!");
				}
					
				int length = stream.readShort();
				System.err.printf("Read Template Set header, ID %d, length %d\n",lastvalue, length);
				
				int bytesread = 4;
				while(bytesread < length){
					int templateID = stream.readShort();
					bytesread += 2;
					int fieldcount = stream.readShort();
					bytesread += 2;
					
					currentTemplate = new Template(templateID, source, (long)observation);
					System.err.printf("Read Template record header, Template ID %d, fields %d\n",templateID, fieldcount);
					for(int i = 0; i < fieldcount; i++){
						int fieldID = stream.readShort();
						bytesread += 2;
						int fieldLength = stream.readShort();
						bytesread += 2;
						int enterpriseID = 0;
						if(fieldID > 0x8000){
							enterpriseID = stream.readInt();
							bytesread += 4;
						}
						Field currentfield = new Field(fieldID, enterpriseID, fieldLength); 
						//enrich with metadata from XML definition
						FieldMetadataHeap.getInstance().updateField(templateID, source, observation, currentfield);
						currentTemplate.addField(currentfield);	
						System.err.printf("Read Template record, fieldID %d, field length %d, enterprise number %d\n",fieldID, fieldLength, enterpriseID);
					}	
					TemplateHeap.getInstance().putTemplate(currentTemplate);
				}
			}

		}else{
			if (stream.available() <= 23){
				throw new Exception("Package of insufficient length, it must be at least 24 Byte to contain a IPFIX Template");
			}
			if(lastvalue != validIPFIXHeader){
				throw new Exception("This parser is for version " + validIPFIXHeader + " of IPFIX, but the value in the header said "+lastvalue);
			}
		}
	}
	
	public static void parseData(byte[] packet, String source)throws Exception{
		Template currentTemplate = null;
		int validIPFIXHeader = 0x000a;
		int lastvalue = 0;

		DataInputStream stream = new DataInputStream(new ByteArrayInputStream(packet));

		if(stream.available() > 16 && (lastvalue = stream.readShort()) == validIPFIXHeader){
			if((lastvalue = stream.readShort()) != packet.length){
				throw new Exception("The specified length does not match the length of the packet. It should be " + lastvalue + " but it is "+ packet.length+ " Byte");
			}

			int export = stream.readInt();
			int sequence = stream.readInt();			
			int observation = stream.readInt();
			LinkedList<Field> fields = new LinkedList<Field>();
			System.err.printf("Read data package with parameters version %d, length %d, export time %d, sequence %d, observation domain %d\n",10, lastvalue, export, sequence, observation);
						
			while(stream.available() > 0){
				if((lastvalue = stream.readShort()) < 256){
					throw new Exception("Datasets must have Set IDs > 255, this one has "+ lastvalue+ "!");
				}
				currentTemplate = TemplateHeap.getInstance().getTemplate(lastvalue, source, observation);
				fields = currentTemplate.getFields();
				int setlength = stream.readShort();

				System.err.printf("Read data set header with ID %d, fields %d\n",lastvalue, setlength);
												
				Field currentField;
				int bytesread = 4;
				while(bytesread < setlength){
					currentField = fields.peekFirst();
					if(sequence <= currentTemplate.getSequence()){
						System.err.println("Got a data set with a obsolete sequence ID, it was ignored.");
						break;
					}
					
					int fieldlen = currentField.getLength();
					if(stream.available() < fieldlen){
						throw new Exception("Field length is longer than the remainder of the packet.");
					}
					bytesread += fieldlen;
					if(currentField.getDatatype().equals("int8")){
						currentField.setValue(stream.readUnsignedByte());
					} else if(currentField.getDatatype().equals("int16")){
						currentField.setValue(stream.readShort());
					}else if (currentField.getDatatype().equals("int32")){
						currentField.setValue(stream.readInt());
					}else if(currentField.getDatatype().equals("int64")){
						currentField.setValue(stream.readLong());
					}else if(currentField.getDatatype().equals("float")){
						currentField.setValue(stream.readFloat());
					}else{
						byte[] buffer = new byte[fieldlen];
						stream.readFully(buffer);
						currentField.setValue(buffer);
					}
					//rotate the list one element further;
					fields.addLast(fields.removeFirst());
					System.err.printf("Read data record \"%s\" with ID %d, enterpriseID %d, length %d. %d byte remain\n",currentField.getName(),currentField.getID(), currentField.getEnterpriseID(),fieldlen, stream.available());
				}
			}

		}else{
			if (stream.available() <= 16){
				throw new Exception("Package of insufficient length, it must be at least 17 Byte to contain a IPFIX Dataset");
			}
			if(lastvalue != validIPFIXHeader){
				throw new Exception("This parser is for version " + validIPFIXHeader + " of IPFIX, but the value in the header said "+lastvalue);
			}
		}
	}	
}
